﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using Tavis.Interfaces;


namespace Maze
{
	public class MazeLinkExtractor : ILinkExtractor
	{
		public IEnumerable<Type> SupportedTypes { get { return new[] { typeof(XmlDocument) }; } }


		public ILink GetLink(ILinkFactory factory, object content, string relation, string anchor = null)
		{
			return GetLinks(factory, content, relation, anchor).FirstOrDefault();
		}


		public IEnumerable<ILink> GetLinks(ILinkFactory factory, object content, string relation = null, string anchor = null)
		{
			if (anchor != null) throw new ArgumentException("Anchor is not supported");

			return GetLinks((XmlDocument)content, factory).Where(l => relation == null || l.Relation == relation);
		}


		private static IEnumerable<ILink> GetLinks(XmlDocument doc, ILinkFactory factory)
		{
			return doc.SelectNodes("//link[@href]")
				.Cast<XmlElement>()
				.Select(e => CreateLink(e, factory))
				.ToList()
			;
		}


		private static ILink CreateLink(XmlElement linkElement, ILinkFactory factory)
		{
			var relname = linkElement.GetAttribute("rel");

			var link = factory.CreateLink(relname);
			link.Target = new Uri(linkElement.GetAttribute("href"), UriKind.RelativeOrAbsolute);

			return link;
		}
	}
}
