﻿using System;
using System.Net;
using System.Net.Http;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Tavis.Http.Tests.Fakes;
using Tavis.Tools;

namespace Tavis {
    [TestClass]
    public class RestAgentTests {

        [TestMethod]
        public void CreateRestAgent() {
            //Arrange
            
            
            //Act
            var agent = new RestAgent();

            //Assert
            Assert.IsNotNull(agent);
        }


        [TestMethod]
        public void ShouldReturnContentFromRootUrl() {
            //Arrange
            var httpClient = new HttpClient(new FakeWebRequestChannel(new StringContent("This is a test")));
            

			var agent = new RestAgent(httpClient);
			agent.SemanticsRegistry.RegisterFormatter(new PlainTextFormatter());
            
            //Act
            agent.NavigateTo(new Link(new Uri("http://example.org/")));

            //Assert
            var textContent = (string)agent.CurrentContent.Value;
            Assert.AreEqual("This is a test", textContent);
            Assert.IsNotNull(agent);
        }



		[TestMethod]
		public void TestErrorHandling()
		{
			// Arrange
			

			var fakeResponse = new HttpResponseMessage(HttpStatusCode.InternalServerError);
			var fakeWebRequestChannel = new FakeWebRequestChannel(fakeResponse);
			
            var httpClient = new HttpClient(fakeWebRequestChannel);
			var agent = new RestAgent(httpClient);
			
			// Act
			agent.NavigateTo(new Link("http://example.org/"));
			var content = agent.CurrentContent;

			// Assert
			Assert.IsNull(content);
			Assert.AreEqual(HttpStatusCode.InternalServerError, agent.CurrentStatusCode);
		}
    }
}
